; -----------------------------------------------------------------------------
;
;  Copyright (C) 1997-2008  Krzysztof M. Gorski, Eric Hivon, Anthony J. Banday
;
;
;
;
;
;  This file is part of HEALPix.
;
;  HEALPix is free software; you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation; either version 2 of the License, or
;  (at your option) any later version.
;
;  HEALPix is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with HEALPix; if not, write to the Free Software
;  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;
;  For more information about HEALPix see http://healpix.jpl.nasa.gov
;
; -----------------------------------------------------------------------------
pro vec2moll, vec, u, v, flip=flip
;+
; VEC2MOLL
;  returns u,v position on Mollweide map corresponding to 3D vector 'vec'
; 
; VEC2MOLL, vec, u, v
;
;-

if keyword_set(flip) then flipconv = +1 else flipconv = -1 ; longitude increase leftward by default (astro convention)
halfpi = !pi / 2.

common vec2mollcom, ntab, vtab, ztab
if undefined(ntab) then begin
    ntab = 180L * 10
    vtab = 2.*FINDGEN(ntab)/(ntab-1) - 1 ; in [-1,1]
    ztab = (ASIN(vtab) + vtab*SQRT(1-vtab^2))/halfpi ; in [-1,1]
endif


np1 = n_elements(vec)/3
vec = reform(vec,np1,3,/Over)

phi = ATAN(vec(*,1), vec(*,0))/ halfpi ; in [-2, 2]

v = INTERPOL(vtab,ztab,vec(*,2)/SQRT(TOTAL(vec^2,2)))
u = flipconv* phi * SQRT(1-v^2) ; minus sign : astro convention


return
end
